/**
 * Test case for apemandocker.
 * Runs with mocha.
 */
'use strict'

const Apemandocker = require('../lib/apemandocker.js')
const assert = require('assert')

describe('apemandocker', function () {
  this.timeout(48000)

  let apemandocker = new Apemandocker({})
  let source = `${__dirname}/../doc/mocks/mock-images/node`
  let image = 'testing-apemandocker-01'
  let container = 'testing-apemandocker-01-container-01'
  let network = 'test-apemandocker-nw-01'

  before((done) => {
    done()
  })

  after((done) => {
    done()
  })

  it('Build an image.', (done) => {
    apemandocker.buildImage(source, {
      tag: image
    }).then(
      () => done(),
      (err) => {
        assert.ifError(err)
        done()
      }
    )
  })

  it('List images.', (done) => {
    apemandocker.listImages({}).then(images => {
      console.log('images:', images)
      assert.ok(images)
      done()
    }).catch((err) => {
      assert.ifError(err)
      done()
    })
  })

  it('Check if image exists.', (done) => {
    apemandocker.imageExists(image)
      .then(exists => {
        assert.ok(exists)
        return exists
      })
      .then(() => {
        return apemandocker.imageExists("__invalid_image_name__")
      })
      .then((exists) => {
        assert.ok(!exists)
        done()
      })
      .catch((err) => {
        assert.ifError(err)
        done()
      })

    ;
  })

  it('Create a container.', (done) => {
    apemandocker.containerExists(container).then(exists => {
      if (exists) {
        done()
        return
      }
      apemandocker.createContainer(image, {
        name: container
      }).then(
        () => done(),
        (err) => {
          assert.ifError(err)
          done()
        }
      )
    })
  })

  it('Check container exists.', (done) => {
    apemandocker
      .containerExists(container)
      .then(exists => {
        assert.ok(exists)
        return exists;
      })
      .then(() => {
        apemandocker.containerExists('__invalid_name__')
      })
      .then((exists) => {
        assert.ok(!exists)
        done()
      })
  })

  it('Start a container.', (done) => {
    apemandocker.startContainer(container, {}).then(() => {
      process.nextTick(() => {
        done()
      })
    }).catch((err) => {
      assert.ifError(err)
      done()
    })
  })

  it('Exec in container.', (done) => {
    apemandocker.execInContainer(container, [ "ls", "-la" ], {}).then(() => {
      process.nextTick(() => {
        done()
      })
    }).catch((err) => {
      assert.ifError(err)
      done()
    })
  })

  it('Check the container running.', (done) => {
    apemandocker.isContainerRunning(container).then(running => {
      assert.equal(running, true)
      done()
    }).catch((err) => {
      assert.ifError(err)
      done()
    })
  })

  it('Log the container.', (done) => {
    apemandocker.logContainer(container).then(() => {
      done()
    }).catch((err) => {
      assert.ifError(err)
      done()
    })
  })

  it('List containers.', (done) => {
    apemandocker.listContainers({
      all: true
    }).then(results => {
      assert.ok(results)
      done()
    }).catch((err) => {
      assert.ifError(err)
      done()
    })
  })

  it('Create network.', (done) => {
    apemandocker.networkExists(network).then((exists) => {
      if (exists) {
        done()
        return
      }
      apemandocker.createNetwork(network, {}).then(() => {
        done()
      }).catch((err) => {
        assert.ifError(err)
        done()
      })
    })
  })

  it('List network.', (done) => {
    apemandocker.listNetworks({}).then((networks) => {
      assert.ok(networks)
      done()
    }).catch((err) => {
      assert.ifError(err)
      done()
    })
  })

  it('Has network.', (done) => {
    apemandocker.networkExists(network).then((exists) => {
      assert.equal(exists, true)
      apemandocker.networkExists("__invalid_network_name__")
        .then(exists => {
          assert.equal(exists, false)
          done()
        })
    }).catch((err) => {
      assert.ifError(err)
      done()
    })
  })

  it('Connect network.', (done) => {
    apemandocker.connectNetwork(network, container, {}).then(() => {
      done()
    }).catch((err) => {
      assert.ifError(err)
      done()
    })
  })

  it('Disconnect network.', (done) => {
    apemandocker.disconnectNetwork(network, container, {}).then(() => {
      done()
    }).catch((err) => {
      assert.ifError(err)
      done()
    })
  })

  it('Terminal with container.', (done) => {
    apemandocker.terminal(container)
      .then(terminal => {

        setTimeout(() => {
          terminal.kill(0)
          done()
        }, 2000)
      })
  })

  it('Inspect the container.', (done) => {
    apemandocker.inspectContainer(container, {}).then((container) => {
      console.log(container)
      done()
    })
  })

  it('Stop the container.', (done) => {
    apemandocker.stopContainer(container, {}).then(
      () => {
        setTimeout(() => {
          done()
        }, 100)
      },
      (err) => {
        assert.ifError(err)
        done()
      }
    )
  })

  it('Remove the container.', (done) => {
    apemandocker.removeContainer(container, {}).then(
      () => done(),
      (err) => {
        assert.ifError(err)
        done()
      }
    )
  })

  it('Remove an image.', (done) => {
    apemandocker.removeImage(image, {
      force: true
    }).then(
      () => done(),
      (err) => {
        assert.ifError(err)
        done()
      }
    )
  })

  it('Remove network.', (done) => {
    apemandocker.removeNetwork(network, {}).then(() => {
      done()
    })
  })

})

/* global describe, before, after, it */
